"""WA Gas Bulletin Board API Wrapper
"""

import requests
import pandas as pd
from pandas.io.json import json_normalize


def __call_api(endpoint):
    return requests.get(f"https://gbbwa.aemo.com.au/api/v1/report/{endpoint}").json()


def _basic_gasday_call(endpoint, gasDay="current", record_prefix=None, column_pop=None):
    result = __call_api(f"{endpoint}/{gasDay}")

    df = pd.io.json.json_normalize(
        result, ["rows"], ["reportId", "asAt", "gasDay"], record_prefix=record_prefix
    )
    if column_pop:
        # The result comes with the Capacity column structured as a dict, so this next line breaks it up to columns
        return df.join(pd.DataFrame(df.pop(column_pop).tolist()))
    return df


def actualFlow(gasDay="current", month=None):
    return _basic_gasday_call("actualFlow", gasDay)


def endUserConsumption(gasDay="current"):
    return _basic_gasday_call("endUserConsumption", gasDay)


def largeUserConsumptionByCategory(gasDay="current"):
    return _basic_gasday_call("largeUserConsumptionByCategory", gasDay)


def mediumTermCapacity(gasDay="current"):
    return _basic_gasday_call("mediumTermCapacity", gasDay)


def gasSpecification(gasDay="current"):
    return _basic_gasday_call("gasSpecification", gasDay, record_prefix="_")


def largeUserConsumption(gasDay="current"):
    return _basic_gasday_call("largeUserConsumption", gasDay, record_prefix="_")


def capacityOutlook(gasDay="current"):
    return _basic_gasday_call("capacityOutlook", gasDay, column_pop="capacity")


def linepackCapacityAdequacy(gasDay="current"):
    return _basic_gasday_call("linepackCapacityAdequacy", gasDay, column_pop="status")


def forecastFlow(gasDay="current"):

    dataFrame_rows = _basic_gasday_call("forecastFlow", gasDay, column_pop="forecast")

    for i in range(7):
        dataFrame_rows = dataFrame_rows.join(
            pd.DataFrame(dataFrame_rows.pop(f"d{i}").tolist())
        )
        dataFrame_rows.rename(
            columns={"delivery": f"d{i}_delivery", "receipt": f"d{i}_receipt"},
            inplace=True,
        )
    return dataFrame_rows



"""
Japan Electric Power eXchange (JEPX)
Website - www.jepx.org
API Details - Day-ahead Spot prices and Trading Volumes over 48 Trading Intervals

Dataset:
Column 1: Date in YYYY/MM/DD format
Column 2: Trading interval (1-48)
Column 3: Price in JPY/kWh
Column 4: Volume in MWh

Usage: call "spotLatest", providing date in YYYYMMDD format. Will otherwise return the latest day-ahead data.


"""

import pandas as pd
from io import StringIO
import requests
from .common import combineTimePeriodDate
from .common import mappingJEPX

def __call_api(endpoint, column_names=None,encoding="utf-8"):
    url = f"http://www.jepx.org/{endpoint}"
    s = StringIO(requests.get(url).content.decode(encoding))
    try:
        if column_names==None:
            return pd.read_csv(s, header=None)
        else:
            return pd.read_csv(s, header=None, names=column_names)
    except:
        return None


def spotLatest(day="SpotLatest"):
    return __call_api(
        f"data/{day}.csv", column_names=["date", "interval", "price", "volume"]
    )

def spotHistorical(year="2019"):
    endpoint =f"market/excel/spot_{year}.csv"
    dfRecords = __call_api(endpoint,encoding="Shift JIS")
    dfRecords = dfRecords[1:]
    dfRecords.rename(columns=mappingJEPX, inplace=True)
    dfRecords = combineTimePeriodDate(dF=dfRecords, targetColumn='DATETIME', periodColumn='PERIOD', targetColumnFmt='%Y/%m/%d')
    return dfRecords







from datetime import timedelta
from datetime import datetime


class commonData:
    dictEiaReports = dict ([
        #region COAL
            #region Production
        ('worldCoalProduction','INTL.4411-1-WORL-QBTU.A'),
        ('africaCoalProduction','INTL.4411-1-AFRC-QBTU.A'),
        ('australiaCoalProduction','INTL.4411-1-AUS-QBTU.A'),
        ('chinaCoalProduction', 'INTL.4411-1-CHN-QBTU.A'),
        ('asiaCoalProduction','INTL.4411-1-ASOC-QBTU.A'),
        ('northAmericaCoalProduction','INTL.4411-1-NOAM-QBTU.A'),
        ('europeCoalProduction', 'INTL.4411-1-EURO-QBTU.A'),

            #endregion Production
            # region Consumption
         ('worldCoalConsumption', 'INTL.4411-2-WORL-QBTU.A'),
         ('africaCoalConsumption', 'INTL.4411-2-AFRC-QBTU.A'),
         ('australiaCoalConsumption', 'INTL.4411-2-AUS-QBTU.A'),
         ('chinaCoalConsumption', 'INTL.4411-2-CHN-QBTU.A'),
         ('asiaCoalConsumption', 'INTL.4411-2-ASOC-QBTU.A'),
         ('northAmericaCoalConsumption', 'INTL.4411-2-NOAM-QBTU.A'),
         ('europeCoalConsumption', 'INTL.4411-2-EURO-QBTU.A'),
         ('southAmericaCoalConsumption', 'INTL.4411-2-CSAM-QBTU.A'),
            # endregion Consumption
        #endregion COAL

        # region NG
        # region Production
        ('worldNGProduction', 'INTL.4413-1-WORL-QBTU.A'),
        ('africaNGProduction', 'INTL.4413-1-AFRC-QBTU.A'),
        ('australiaNGProduction', 'INTL.4413-1-AUS-QBTU.A'),
        ('chinaNGProduction', 'INTL.4413-1-CHN-QBTU.A'),
        ('asiaNGProduction', 'INTL.4413-1-ASOC-QBTU.A'),
        ('northAmericaNGProduction', 'INTL.4413-1-NOAM-QBTU.A'),
        ('europeNGProduction', 'INTL.4413-1-EURO-QBTU.A'),

        # endregion Production
        # region Consumption
        ('worldNGConsumption', 'INTL.4413-2-WORL-QBTU.A'),
        ('africaNGConsumption', 'INTL.4413-2-AFRC-QBTU.A'),
        ('australiaNGConsumption', 'INTL.4413-2-AUS-QBTU.A'),
        ('chinaNGConsumption', 'INTL.4413-2-CHN-QBTU.A'),
        ('asiaNGConsumption', 'INTL.4413-2-ASOC-QBTU.A'),
        ('northAmericaNGConsumption', 'INTL.4413-2-NOAM-QBTU.A'),
        ('europeNGConsumption', 'INTL.4413-2-EURO-QBTU.A'),
        ('southAmericaNGConsumption', 'INTL.4413-2-CSAM-QBTU.A'),
        # endregion Consumption

        # region Price
        ('henryHubNGSpotPriceDaily', 'NG.RNGWHHD.D'),
        # endregion Price
        # endregion NG

        # region CRUDE
        # region Price
        ('wtiCushingSpotPriceDaily', 'PET.RWTC.D'),
        # endregion Price
        # endregion CRUDE

    ])


def combineTimePeriodDate(dF,targetColumn='DATETIME',periodColumn='PERIOD',targetColumnFmt ='%Y/%m/%d' ):
    for i, row in dF.iterrows():
        minutes_add = ((int(dF.at[i, periodColumn]) * 30) - 30)
        dF.at[i, targetColumn] = datetime.strptime(dF.at[i, targetColumn], targetColumnFmt) + timedelta(minutes=int(minutes_add))
    dF.set_index(targetColumn, inplace=True)
    return dF


mappingJEPX = ({0:'DATETIME'
,1:'PERIOD'
,2:'Selling bid amount (kWh)'
,3:'buying bid amount (kWh)'
,4:'execution total amount (kWh)'
,5:'system price (JPY/kWh)'
,6:'Area price Hokkaido (JPY/kWh)'
,7:'Area price Tohoku (JPY/kWh)'
,8:'Area price Tokyo (JPY/kWh)'
,9:'Area Price Chubu (JPY/kWh)'
,10:'Area Price Hokuriku (JPY/kWh)'
,11:'Area Price Kansai (JPY/kWh)'
,12:'Area Price Chuugoku (JPY/kWh)'
,13:'Area Price Shikoku (JPY/kWh)'
,14:'Area price Kyushu (JPY/kWh)'
,15:'spot · time before average price (JPY/kWh)'
,16:'upper limit value × spot time before average price (JPY/kWh) '
,17:'lower limit value × spot · time before average price (JPY/kWh)'
,18:'preliminary value × spot · time before average price (JPY/kWh)'
,19:'confirmed value × spot · time before average price (JPY/kWh)'
,20:'avoidable cost national price (JPY/kWh)'
,21:'avoidable Cost Hokkaido (JPY/kWh)'
,22:'avoidable cost Tohoku (JPY/kWh)'
,23:'avoidable cost Tokyo (JPY/kWh)'
,24:'avoidable cost Chubu (JPY/kWh)'
,25:'avoidable cost Hokuriku (JPY/kWh)'
,26:'avoidable Cost Kansai (JPY/kWh)'
,27:'avoidable Cost Chuugoku (JPY/kWh)'
,28:'avoid Noh cost Shikoku (JPY/kWh)'
,29:'avoidable cost Kyushu (JPY/kWh)'
                        })


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="energy_trading_api",
    version="0.0.28",
    author="jericmac",
    author_email="",
    description="API Wrappers for the Energy Markets",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jericmac/energy-trading-api-wrappers",
    install_requires=["eia-python == 1.22","Scrapy==1.6.0"],
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


"""
Western Australia Wholesale Electricity Market (WEM) Web APIs

An API Wrapper for the reports found in the WEM Data website:
    http://data.wa.aemo.com.au

    -Participants
    -Facilities
    -Comissioning Tests
    -Load Forecast
    -Extended Load Forecast
    -Historical Load Forecast
    -Non Balancing Dispatch Merit Order
    -Demand SIde Programme Prices
    -Operational Measurements
    -Facility SCADA
    -Load Summary
    -Outages
    -IRCR Ratios
    -Peak SWIS Trading Intervals
    -Real Time Outages
    -Refund Exempt Planned Outage
    -Real Time Dispatch Advisories
    -Market Advisories
    -Balancing Market Summary
    -Effective Balancing Submission
    -STEM Summary
    -STEM Participant Activity
    -STEM Bids and Offers
    -STEM Facility Declarations

"""

import pandas as pd
from io import StringIO
import requests
import datetime

def __call_api(endpoint, ignore_errors=False,low_memory=True):
    url = f"http://data.wa.aemo.com.au/datafiles/{endpoint}"
    try:
        if ignore_errors==False:
            s = StringIO(requests.get(url).content.decode("utf-8"))
        else:
            s = StringIO(requests.get(url).content.decode("utf-8","ignore"))

        if low_memory==True:
            return pd.read_csv(s)
        else:
            return pd.read_csv(s,low_memory=False)
    except Exception as e:
        print(e)
        return None

def participants():
    return __call_api(endpoint=f"participants/participants.csv")

def facilities():
    return __call_api(endpoint=f"facilities/facilities.csv")

def commissioningTest(year="2019"):
    return __call_api(endpoint=f"commissioning-test/commissioning-test-summary-{year}.csv")

def loadForecast():
    return __call_api(endpoint=f"load-forecast/load-forecast.csv")

def extendedLoadForecast():
    return __call_api(endpoint=f"extended-load-forecast/extended-load-forecast.csv")

def historicalLoadForecast(year="2019"):
    return __call_api(endpoint=f"historical-load-forecast/historical-load-forecast-{year}.csv")

def nonbalancingDispatchMeritOrder(yearmonth="2019-01"):
    return __call_api(endpoint=f"non-balancing-dispatch-merit-order/nbdmo-{yearmonth}.csv")

def demandSideProgrammePrices(year="2019"):
    return __call_api(endpoint=f"dsp-decrease-price/dsp-decrease-price-{year}.csv")

def operationalMeasurements(year="2019"):
    return __call_api(endpoint=f"operational-measurements/operational-measurements-{year}.csv")

def facilitySCADA(yearmonth="2019-01"):
    return __call_api(endpoint=f"facility-scada/facility-scada-{yearmonth}.csv",low_memory=False)

def loadSummary(year="2018"):
    return __call_api(endpoint=f"load-summary/load-summary-{year}.csv")

def outages(year="2019"):
    return __call_api(endpoint=f"outages/outages-{year}.csv",low_memory=False)

def ircrRatios():
    return __call_api(endpoint=f"ircr-ratios/ircr-ratios.csv")

def peakSWISIntervals():
    return __call_api(endpoint=f"peak-intervals/peak-intervals.csv")

def realTimeOutages():
    return __call_api(endpoint=f"realtime-outages/realtime-outages.csv",ignore_errors=True)

#REPO = refund exempt planned outage count
def repoCount():
    return __call_api(endpoint=f"repo-count/repo-count.csv")

def dispatchAdvisory():
    return __call_api(endpoint=f"dispatch-advisory/dispatch-advisory.csv",ignore_errors=True)

def marketAdvisory():
    return __call_api(endpoint=f"market-advisory/market-advisory.csv")

def balancingSummary(year="2019"):
    return __call_api(endpoint=f"balancing-summary/balancing-summary-{year}.csv")

def effectiveBalancingSubmission(yearmonth="2019-01"):
    return __call_api(endpoint=f"effective-balancing-submission/effective-balancing-submission-{yearmonth}.csv",low_memory=False)

def stemSummary(year="2019"):
    return __call_api(endpoint=f"stem-summary/stem-summary-{year}.csv")

def stemParticipantActivity(yearmonth="2019-01"):
    return __call_api(endpoint=f"stem-participant-activity/stem-participant-activity-{yearmonth}.csv")

def stemBidsAndOffers(yearmonth="2019-01"):
    return __call_api(endpoint=f"stem-bids-and-offers/stem-bids-and-offers-{yearmonth}.csv",low_memory=False)

def stemFacilityDeclarations(yearmonth="2019-01"):
    return __call_api(endpoint=f"stem-facility-declarations/stem-facility-declarations-{yearmonth}.csv",low_memory=False)
